/*
 * Copyright 2008 Adam Tacy <adam.tacy AT gmail.com>
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.adamtacy.client.ui.effects;

import java.util.Iterator;
import java.util.Vector;

import org.adamtacy.client.ui.NEffectPanel;
import org.adamtacy.client.ui.effects.transitionsphysics.TransitionPhysics;

/**
 * 
 * Simple Collection object to hold Effects
 * 
 */
@SuppressWarnings("serial")
public class NEffectCollection extends Vector<NEffect> {

  /**
   * Invert the effects.
   */
  public void invertEffects() {
    for (Iterator<NEffect> it = iterator(); it.hasNext();)
      it.next().invert();
  }

  /**
   * Play the effects registered with a particular EffectPanel from start
   * position to the end position
   */
  public void playEffects() {
    for (Iterator<NEffect> it = iterator(); it.hasNext();)
      it.next().play();
  }

  /**
   * Play the effects registered with a particular EffectPanel from a start
   * position to an end position
   */
  public void playEffects(double start, double end) {
    for (Iterator<NEffect> it = iterator(); it.hasNext();)
      it.next().play(start, end);
  }

  public void removeEffects() {
    for (Iterator<NEffect> it = iterator(); it.hasNext();)
      it.next().remove();
  }

  public void resumeEffectsBackwards() {
    for (Iterator<NEffect> it = iterator(); it.hasNext();)
      it.next().resumeBackwards();
  }
  
  public void resumeEffectsBackwards(double end) {
    for (Iterator<NEffect> it = iterator(); it.hasNext();)
      it.next().resumeBackwards(end);
  }
  
  public void resetEffects() {
    for (Iterator<NEffect> it = iterator(); it.hasNext();)
      it.next().reset();
  }

  public void resumeEffectsForwards() {
    for (Iterator<NEffect> it = iterator(); it.hasNext();)
      it.next().resumeForwards();
  }
  
  public void resumeEffectsForwards(double end) {
    for (Iterator<NEffect> it = iterator(); it.hasNext();)
      it.next().resumeForwards(end);
  }

  /**
   * Sets the effect duration (in seconds) for all effects on the Panel
   * 
   * @param duration The new duration that should be applied to all effects on
   *          the panel.
   */
  public void setEffectsLength(double duration) {
    for (Iterator<NEffect> it = iterator(); it.hasNext();) {
      NEffect theEffect = it.next();
      if (theEffect instanceof SequentialCompositeEffect)
        ((SequentialCompositeEffect) theEffect).setDuration(duration);
      else if (theEffect instanceof ParallellCompositeEffect)
        ((ParallellCompositeEffect) theEffect).setDuration(duration);
      else
        theEffect.setDuration(duration);
    }
  }
  
  
  /**
   * Allow external user to specify position within an effect
   * @param progress the position to be set.
   */
  public void setPosition(double position) {
    for (Iterator<NEffect> it = iterator(); it.hasNext();)
      it.next().setPosition(position);
  }

  /**
   * Sets the transition type for all effects on the Panel
   * 
   * @param newTransition The new Transition that should be applied to all
   *          effects on the panel.
   */
  public void setTransitionTypes(TransitionPhysics newTransition) {
    for (Iterator<NEffect> it = iterator(); it.hasNext();) {
      NEffect theEffect = it.next();
      if (theEffect instanceof SequentialCompositeEffect)
        ((SequentialCompositeEffect) theEffect).setTransitionType(newTransition);
      else if (theEffect instanceof ParallellCompositeEffect)
        ((ParallellCompositeEffect) theEffect).setTransitionType(newTransition);
      else
        theEffect.setTransitionType(newTransition);
    }
  }

  /**
   * Begin the effects registered with a particular EffectPanel.
   */
  public void setUpEffects(NEffectPanel thePanel) {
    for (Iterator<NEffect> it = iterator(); it.hasNext();) {
      NEffect theEffect = it.next();
      if (theEffect instanceof SequentialCompositeEffect)
        ((SequentialCompositeEffect) theEffect).init(thePanel);
      else if (theEffect instanceof ParallellCompositeEffect)
        ((ParallellCompositeEffect) theEffect).init(thePanel);
      else
        theEffect.init(thePanel);
    }
  }

  public void stopEffects() {
    for (Iterator<NEffect> it = iterator(); it.hasNext();)
      it.next().cancel();
  }

  /**
   * Begin the effects registered with a particular EffectPanel.
   */
  public void tearDownEffects() {
    for (Iterator<NEffect> it = iterator(); it.hasNext();) {
      NEffect theEffect = it.next();
      if (theEffect instanceof SequentialCompositeEffect)
        ((SequentialCompositeEffect) theEffect).tearDownEffect();
      else if (theEffect instanceof ParallellCompositeEffect)
        ((ParallellCompositeEffect) theEffect).tearDownEffect();
      else
        theEffect.tearDownEffect();
    }
  }
}