/*
 * Copyright 2008 Adam Tacy <adam.tacy AT gmail.com>
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.adamtacy.client.ui.effects;

import java.util.Iterator;

import org.adamtacy.client.ui.NEffectPanel;
import org.adamtacy.client.ui.effects.transitionsphysics.TransitionPhysics;

/**
 * Extend this class if you want to provide the user a new Effect which is
 * composed from other already defined Effects which you wish the user to
 * execute in parallel. As a programmer, you can add parallel events to a panel
 * using the existing methods and effectHandlers.
 * 
 * Setting the duration of this composite effect has affect of dividing the
 * duration equally amongst all of the effects in the sequence.
 * 
 * Additionally, it is possible to set a delay between the sequencing of the
 * events.
 * 
 * This class is defined is abstract since it is not expected that the user can
 * create an Effect, rather, then must create instances of Effect subclasses
 * which implement a specific effect.
 * 
 */
public abstract class ParallellCompositeEffect extends NEffect {

  /**
   * An internal collection of effects that are added
   */
  protected NEffectCollection internalEffects = new NEffectCollection();

  private double duration = 10;

  private int numberEffectsRegistered = 0;
  private boolean initialized = false;

  /**
   * Inner class representing the properties that may be altered by the user.
   * @deprecated
   */
  public class BaseParallelProperties extends BaseProperties {
    /**
     * @deprecated
     */
    public void setEffectLength(double new_EffectLengthSeconds) {
      setParallelEffectLength(new_EffectLengthSeconds);
    }
  }
  
  /**
   * @deprecated
   */
  public void setEffectLength(double new_EffectLengthSeconds) {
    setDuration(new_EffectLengthSeconds);
  }
  
  public void setDuration(double new_EffectLengthSeconds) {
    setParallelEffectLength(new_EffectLengthSeconds);
  }
  
  public void setTransitionType(TransitionPhysics newTP){
    internalEffects.setTransitionTypes(newTP);
  }

  protected void finishCompositeEffect() {
    if (effectHandlers != null)
      effectHandlers.finishedEffect(this);
  }

  @Override
  protected void setUpEffect() {
    for (Iterator<NEffect> it = internalEffects.iterator(); it.hasNext();)
      ((NEffect) it).setUpEffect();
  }

  @Override
  public void tearDownEffect() {
    internalEffects.tearDownEffects();
  }
  
  
  protected void finished() {
    finishCompositeEffect();
  }

  protected void resetCompositeEffect() {
  }

  public void reset() {
    for (Iterator<NEffect> it = internalEffects.iterator(); it.hasNext();)
      ((ParallellCompositeEffect) it).reset();
    resetCompositeEffect();
  }

  public void beginCompositeEffect() {
  }

  public void play(double start, double end) {
    if (numberEffectsRegistered < 2)
      throw new RuntimeException(
          "A composite effect must have more than one effect registered");
    if (!initialized)
      throw new RuntimeException(
          "A composite effect must be initialized before attempting to use");
    beginCompositeEffect();
    if (effectHandlers != null)
      effectHandlers.startedEffect(this);
    for (Iterator<NEffect> it = internalEffects.iterator(); it.hasNext();) {
      NEffect theEffect = (NEffect) it.next();
      theEffect.play(start, end);
    }
  }
  
  public void play(){
    play(0.0, 1.0);
  }

  public void registerEffect(NEffect e) {
    internalEffects.add(e);
    numberEffectsRegistered++;
  }

  public void init(NEffectPanel panel) {
    thePanel = panel;
    for (Iterator<NEffect> it = internalEffects.iterator(); it.hasNext();) {
      NEffect theEffect = (NEffect) it.next();
      theEffect.init(panel);
    }
    effectElement = panel.getElement();
    initialized = true;
  }

  private void setParallelEffectLength(double seconds) {
    duration = seconds;
    for (Iterator<NEffect> it = internalEffects.iterator(); it.hasNext();) {
      NEffect theEffect = (NEffect) it.next();
      theEffect.setDuration(duration);
    }
  }
  

  public void addEffectHandler(NEffectHandler eff){
    internalEffects.get(0).addEffectHandler(eff);
  }
  

  public void initCompositeEffect() {
    initialized = true;
  }

  public ParallellCompositeEffect() {
    super();
  }
  
}