/*
 * Copyright 2008 Adam Tacy <adam.tacy AT gmail.com>
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.adamtacy.client.ui.effects;

import java.util.Iterator;

import org.adamtacy.client.ui.NEffectPanel;
import org.adamtacy.client.ui.effects.transitionsphysics.TransitionPhysics;

import com.google.gwt.user.client.Timer;

/**
 * Extend this class if you want to provide the user a new Effect which is
 * composed from other already defined Effects. As a programmer, you can add
 * sequential events to a panel using the existing methods and effectHandlers.
 * 
 * Setting the duration of this composite effect has affect of dividing the
 * duration equally amongst all of the effects in the sequence.
 * 
 * Additionally, it is possible to set a delay between the sequencing of the
 * events.
 * 
 * This class is defined is abstract since it is not expected that the user can
 * create an Effect, rather, then must create instances of Effect subclasses
 * which implemenet a specific effect.
 * 
 */
public abstract class SequentialCompositeEffect extends NEffect {

  /**
   * Inner class representing the properties that may be altered by the user.
   * @deprecated
   */
  public class BaseSequentialProperties extends BaseProperties {
    
    /**
     * @deprecated
     * @param new_EffectDelayLengthSeconds
     */
    public void setEffectDelayLength(double new_EffectDelayLengthSeconds) {
      setEffectBetweenDelay(new_EffectDelayLengthSeconds);
    }

    /**
     * @deprecated
     */
    @Override
    public void setEffectLength(double new_EffectLengthSeconds) {
      setSequentialEffectLength(new_EffectLengthSeconds);
    }
  }
  
  /**
   * @param new_EffectDelayLengthSeconds
   */
  public void setEffectDelayLength(double new_EffectDelayLengthSeconds) {
    setEffectBetweenDelay(new_EffectDelayLengthSeconds);
  }
  
  public void setDuration(double new_EffectDelayLengthSeconds) {
    setSequentialEffectLength(new_EffectDelayLengthSeconds);
  }

  /**
   * @deprecated
   */
  @Override
  public void setEffectLength(double new_EffectLengthSeconds) {
    setDuration(new_EffectLengthSeconds);
  }
  

  private int delayBetweenEffect = 0;

  private double duration = 1.0;
  private boolean initialized = false;
  /**
   * An internal collection of effects that are added
   */
  protected NEffectCollection internalEffects = new NEffectCollection();

  private int numberEffectsRegistered = 0;

  public SequentialCompositeEffect() {
    super();
  }

  public void beginCompositeEffect() {
  }

  private void beginNextEffect(NEffect e) {
    if (e instanceof SequentialCompositeEffect)
      ((SequentialCompositeEffect) e).play();
    else if (e instanceof ParallellCompositeEffect)
      ((ParallellCompositeEffect) e).play();
    else
      e.play();
  }

  public void clearEffects() {
    for (Iterator<NEffect> it = internalEffects.iterator(); it.hasNext();) {
      thePanel.removeEffect(it.next());

    }
    numberEffectsRegistered = 0;
    internalEffects.clear();
  }

  protected void finishCompositeEffect() {
    if (effectHandlers != null)
      effectHandlers.finishedEffect(this);
  }

  protected void finished() {
    finishCompositeEffect();
  }

  @Override
  public void init(NEffectPanel panel) {
    effectElement = panel.getPanelWidget().getElement();
    thePanel = panel;
    for (Iterator<?> it = internalEffects.iterator(); it.hasNext();) {
      Object theEffect = it.next();
      if (theEffect instanceof SequentialCompositeEffect)
        ((SequentialCompositeEffect) theEffect).init(panel);
      else if (theEffect instanceof ParallellCompositeEffect)
        ((ParallellCompositeEffect) theEffect).init(panel);
      else if (theEffect instanceof NEffect)
        ((NEffect) theEffect).init(panel);
      else
        throw new RuntimeException("Not an Effect");
    }
    initialized = true;
  }

  public void initCompositeEffect() {
    for (int loop = 0; loop < internalEffects.size() - 1; loop++) {
      NEffect firstEffect = (internalEffects.get(loop));
      final NEffect secondEffect = (internalEffects.get(loop + 1));
      firstEffect.addEffectHandler(new NEffectHandler() {
        public void interruptedEvent(NEffect theEffect) {
        }

        public void postEvent(NEffect theEffect) {
          if (delayBetweenEffect > 0) {
            Timer t = new Timer() {
              @Override
              public void run() {
                beginNextEffect(secondEffect);
              }
            };
            t.schedule(delayBetweenEffect);
          } else {
            beginNextEffect(secondEffect);
          }
        }

        public void preEvent(NEffect theEffect) {
        }

        public void duringEvent(NEffect theEffect) {          
        }

      });
    }
    (internalEffects.get(internalEffects.size() - 1)).addEffectHandler(new NEffectHandler() {
      public void interruptedEvent(NEffect theEffect) {
      }

      public void postEvent(NEffect theEffect) {
        finishCompositeEffect();
      }

      public void preEvent(NEffect theEffect) {
      }

      public void duringEvent(NEffect theEffect) {        
      }
    });
    this.setEffectLength();
    initialized = true;
  }

  @Override
  public void play() {
    if (numberEffectsRegistered < 2)
      throw new RuntimeException(
          "A composite effect must have more than one effect registered");
    if (!initialized)
      throw new RuntimeException(
          "A composite effect must be initialized before attempting to use");
    beginCompositeEffect();
    if (effectHandlers != null)
      effectHandlers.startedEffect(this);
    Object theEffect = internalEffects.get(0);
    if (theEffect instanceof SequentialCompositeEffect)
      ((SequentialCompositeEffect) theEffect).play();
    else if (theEffect instanceof ParallellCompositeEffect)
      ((ParallellCompositeEffect) theEffect).play();
    else
      ((NEffect) theEffect).play();
  }

  public void registerEffect(NEffect e) {
    internalEffects.add(e);
    numberEffectsRegistered++;
  }

  public void registerEffect(ParallellCompositeEffect e) {
    internalEffects.add(e);
    numberEffectsRegistered++;
  }

  public void registerEffect(SequentialCompositeEffect e) {
    internalEffects.add(e);
    numberEffectsRegistered++;
  }

  @Override
  public void reset() {
    // ((NEffect) internalEffects.get(0)).reset();
    resetCompositeEffect();
  }

  protected void resetCompositeEffect() {
  }

  private void setEffectBetweenDelay(double seconds) {
    delayBetweenEffect = new Double(seconds * 1000).intValue();
  }

  private void setEffectLength() {
    this.setSequentialEffectLength(duration);
  }

  private void setSequentialEffectLength(double seconds) {
    duration = seconds;
    double individualEffectLength = seconds / numberEffectsRegistered;
    for (Iterator<NEffect> it = internalEffects.iterator(); it.hasNext();) {
      NEffect theEffect = it.next();
      theEffect.setDuration(individualEffectLength);
    }
  }

  @Override
  public void setTransitionType(TransitionPhysics newTransition) {
    for (Iterator<NEffect> it = internalEffects.iterator(); it.hasNext();) {
      NEffect theEffect = it.next();
      theEffect.setTransitionType(newTransition);
    }
  }
}