/*
 * Copyright 2007 Adam Tacy <adam.tacy AT gmail.com>
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.adamtacy.client.ui.effects.core;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import org.adamtacy.client.ui.effects.NEffect;
import org.adamtacy.client.ui.effects.impl.css.StyleImplementation;
import org.adamtacy.client.ui.effects.impl.css.Rule;
import org.adamtacy.client.ui.effects.impl.css.Property;
import org.adamtacy.client.ui.effects.impl.css.Selector;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.ui.Widget;

public class NMorphStyle extends NEffect {

  /**
   * Inner class representing a pair of properties start can be null, but the
   * end should not be
   * 
   * @author adam
   * 
   */
  protected class PropertyPair {
    protected Property start, end;
    
    public Property getStart(){
      return start;
    }
    
    public Property getEnd(){
      return end;
    }

    public PropertyPair(Property start, Property end) {
      this.start = start;
      this.end = end;
      if (end == null)
        throw new RuntimeException(
            "End value of a property pair shold not be null");
    }
  }

  /**
   * Property class of the Effect
   * 
   * @author adam
   * @deprecated
   */
  public class StyleProperties extends BaseProperties {
    
    /** @deprecated
     * 
     * @param start
     */
    public void setNewStartStyle(Rule start){
      registerProperties(start, theEnd);
      theStart = start;
    }
    
    /**
     * @deprecated
     * @param end
     */
    public void setNewEndStyle(Rule end){
      registerProperties(theStart, end);
      theEnd = end;
    }
    
    /**
     * @deprecated
     * @param start
     * @param end
     */
    public void setNewStyles(Rule start, Rule end){
      registerProperties(start,end);
      theStart = start;
      theEnd = end;
    }
  }
  
  public void setNewStartStyle(Rule start){
    registerProperties(start, theEnd);
    theStart = start;
    if ((thePanel!=null) && thePanel.isAttached()) setUpEffect();
  }
  
  public void setNewEndStyle(Rule end){
    registerProperties(theStart, end);
    theEnd = end;
    if ((thePanel!=null) && thePanel.isAttached()) setUpEffect();
  }
  
  public void setNewStyles(Rule start, Rule end){
    registerProperties(start,end);
    theStart = start;
    theEnd = end;
    if ((thePanel!=null) && thePanel.isAttached()) setUpEffect();
  }

  /**
   * Set of effects that it has been determined make up the morph from the start
   * to the end styles supplied in the constructor.
   */
  protected HashMap<String, ChangeInterface> determinedEffects; 
  
  /**
   * Set of property pairs representing the valid start/end pairs of properties
   * the effect will work over.
   */
  protected Vector<PropertyPair> propertyPairs;

  protected Rule theStart, theEnd;

  /**
   * Internal copy of the properties object
   * @deprecated
   */
  private StyleProperties theStyleProperties = new StyleProperties();

  /**
   * Constructor to morph a single Property between two values
   * 
   * @param start Initial property.
   * @param end Expected end property.
   */
  public NMorphStyle(Property start, Property end) {
    if (!start.getName().equals(end.getName()))
      throw new RuntimeException(
          "Start and End properties must be of the same type!  You used start="
              + start.getName() + " and end=" + end.getName());
    propertyPairs.add(new PropertyPair(start, end));
  }

  /**
   * Constructor to morph between two style rules
   * 
   * @param start Initial style rule.
   * @param end Expected end style rule.
   */
  public NMorphStyle(Rule start, Rule end) {
    registerProperties(start, end);
    theStart = start;
    theEnd = end;
  }

  /**
   * Constructor to morph between two style rules defined in an external style
   * sheet.
   * 
   * @param start Initial style rule selector.
   * @param end Expected end style rule selector.
   */
  public NMorphStyle(final Selector start, final Selector end) {
    theStart = start.getRuleDefinition();
    theEnd = end.getRuleDefinition();
    registerProperties(theStart, theEnd);
  }

  public NMorphStyle() {
  }
  
  public NMorphStyle(String w) {
    this();
    applyEffect(w,this);
  }
  
  public NMorphStyle(Widget w){
    this();
    applyEffect(w,this);
  }

  /**
   * Allow the user to retrieve a copy of the properties object.
   * 
   * @return ColourProperties object
   * @deprecated
   */
  public StyleProperties getProperties() {
    return theStyleProperties;
  }
  
  public void setPosition(double progress){
    double val = interpolate(progress);
    this.onUpdate(val);
  }

  @Override
  protected void onUpdate(double progress) {
    super.onUpdate(progress);
    for (Iterator<String> it = determinedEffects.keySet().iterator(); it.hasNext();) {
      String theChange = it.next();
      ChangeInterface currChange = determinedEffects.get(theChange);
      currChange.performStep(effectElement, theChange, progress);
    }
  }

  /**
   * Creates a list of properties that we are interested in mapping between. In
   * the worst case, this is all properties provided in the rules, however there
   * are two spcial cases:
   * 
   * 1. Items that appear in the start rule but not in the end rule. The
   * understanding here is that this property will not change over the
   * progression, so we can ignore it. 2. Items that appear in the end rule but
   * not in the start rule. The understanding here is that we need to progress
   * from some as yet undefined state to the end state. When the effect is set
   * up, an attempt is made to get the computed style for the start state
   * (failing if the start style is determine to be "auto" since how is that to
   * be interpreted.
   * 
   * @param start
   * @param end
   */
  protected void registerProperties(Rule start, Rule end) {
    propertyPairs  = new Vector<PropertyPair>();
    // Since we are interested in end properties, iterate over them.
    for (Iterator<String> it = end.getProperties().keySet().iterator(); it.hasNext();) {
      // Get the propety name
      String name = it.next();
      // Find the entry in the start rule - this could be null.
      Property entryStart = start.getProperties().get(name);
      // Find the entry in the end rule - this can not be null.
      Property entryEnd = end.getProperties().get(name);
      // Create and record a property pair
      propertyPairs.add(new PropertyPair(entryStart, entryEnd));
    }
  }

  /**
   * Sets up the morph effect. Take the previously calculated set of property
   * pairs and for each one ensure there are valid start and end values (which
   * might involve calculating the start value from the attached element - which
   * is why this step is done in this method rather than in the constructor.
   * 
   * With start and end values found, determine and register the appropriate
   * Change object to use (ie scalar, binary, color, clip) to inact the morph
   * for this property.
   * 
   */
  @Override
  protected void setUpEffect() {
    //assert thePanel.getPanelWidget().getElement() != null;

    // return if we have already done this (identifiable if property pairs is
    // null)
    if (propertyPairs == null)
      return;

    double switchFrameNumber = 0.5;

    effectElement = thePanel.getPanelWidget().getElement();
    
    determinedEffects = new HashMap<String, ChangeInterface>();

    // Iterate over the property pairs to ensure values are set.
    for (Iterator<PropertyPair> it = propertyPairs.iterator(); it.hasNext();) {
      try {
        PropertyPair theMorph = it.next();
        String styleName = theMorph.end.getName();
        Property start = theMorph.start;
        Property end = theMorph.end;
        if (start == null) {
          start = new Property(styleName
              + ":"
              + StyleImplementation.getComputedStyle(
                  thePanel.getPanelWidget().getElement(), styleName));
        }
        // Now determine that type of change that is required.
        ChangeInterface theChange;
        if (StyleImplementation.isColour(start.toString())) {
          theChange = new NChangeColorAction(start.getValue(), end.getValue());
          determinedEffects.put(styleName, theChange);
          theChange.setUp(effectElement, styleName, switchFrameNumber);
        } else if (StyleImplementation.isScalar(start.toString(),
            start.getUnitizedValue())) {
          theChange = new NChangeScalarAction(start.getValue()
              + start.getUnits(), end.getValue() + end.getUnits());
          determinedEffects.put(styleName, theChange);
          theChange.setUp(effectElement, styleName, switchFrameNumber);
        } else
        if (StyleImplementation.isClip(start.toString())) {
          theChange = new NChangeClipAction(
              start.getUnitizedValue(), end.getUnitizedValue());
          determinedEffects.put(styleName, theChange);
          theChange.setUp(effectElement, styleName, switchFrameNumber);
        } else { // Assume its a binary style
          GWT.log(
              "Assuming that "
                  + start.getName()
                  + " is a style attribute that switches as a binary (have you forgotten to include the unit (px/pt/em etc)?",
              null);
          theChange = new NChangeBinaryAction(start.getValue(), end.getValue());
          determinedEffects.put(styleName, theChange);
          theChange.setUp(effectElement, styleName, switchFrameNumber);
        }
      } catch (RuntimeException e) {
        GWT.log("MorphEffect CSS Style Exception", e);
      }
    }
    // Finally call reset() which applies the start style to the element.
    reset();
    // and we can get rid of the property pairs element since it has now done
    // its job.
    propertyPairs = null;

    // Assign the start style to the element
    // CAUSED AN ERROR WITH IE (surprisingly) where the
    // getElementAttribute() method in GWT seems to not
    // return back a String.
    //
    // Would appear to be fixed in GWT 1.5 (release) : http://groups.google.com/group/Google-Web-Toolkit-Contributors/browse_thread/thread/c708f46d592bcad8#
    //
//    if(GWT.isScript()){
//    DOM.setElementAttribute((com.google.gwt.user.client.Element)effectElement,
//     "style", DOM.getElementAttribute(
//     (com.google.gwt.user.client.Element)effectElement, "style") +
//      theStart.getDefinition());
//    }
//    
    
      }

  @Override
  public void tearDownEffect() {
    for (Iterator<String> it = determinedEffects.keySet().iterator(); it.hasNext();) {
      ChangeInterface currChange = determinedEffects.get(it.next());
      currChange.tearDownEffect(effectElement);
    }
  }
}
